﻿using System;
using System.Threading;
using System.Windows.Forms;

namespace PI
{
    /// <summary>
    /// Controller
    /// </summary>
    public class Controller
    {
        private int end;
        private MyForm parent;
        private string[] result;
        private double time;
        private Thread[] threads;

        private System.Timers.Timer threadTimer;

        /// <summary>
        /// Initialisiert eine neue Instanz
        /// </summary>
        /// <param name="parent">die Form</param>
        /// <param name="end">Anzahl der zu berechnenden Stellen</param>
        public Controller(MyForm parent, int end)
        {
            this.end = end;
            this.parent = parent;
            this.threadTimer = new System.Timers.Timer(1000);
            this.threadTimer.Elapsed += new System.Timers.ElapsedEventHandler(threadTimer_Elapsed);
        }

        /// <summary>
        /// Überprüft jede Sekunde, ob die Berechnung abgeschlossen wurde.
        /// Anschließend wird das Ergebnis ausgegeben.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void threadTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            foreach (Thread t in this.threads) if (t.IsAlive) return;
            this.threadTimer.Stop();
            foreach(string s in this.result) this.parent.AppendOutBoxText(s);
            this.parent.ToggleButtons();
        }
        /// <summary>
        /// Bricht die Berechnung ab
        /// </summary>
        public void Abort()
        {
            foreach (Thread t in this.threads) t.Abort();
        }
        /// <summary>
        /// Startet die Berechnung.
        /// </summary>
        /// <param name="mt">true, wenn Multithreading, sonst false</param>
        public void StartCalculation(bool mt)
        {
            int count;
            if (mt)
                count = 2;
            else
                count = 1;

            this.threads = new Thread[count];
            this.result = new string[this.end / 9];
            for (int i = 0; i < this.threads.Length; i++)
            {
                this.threads[i] = new Thread(new ThreadStart(this.CalcThread));
                this.threads[i].SetApartmentState(ApartmentState.MTA);
                this.threads[i].Name = i.ToString();
                this.threads[i].Start();
            }

            this.threads[0].Priority = ThreadPriority.Highest;
            this.threadTimer.Start();
        }

        private void CalcThread()
        {
            PI pi = new PI();
            Timer timer = null;

            // Die Threadnummer holen wir uns aus dem Namen
            int i = int.Parse(Thread.CurrentThread.Name);
            // Bei zwei Threads sieht es so aus:
            // step = 18
            // 1. Thread: start = 0, pos = 0
            // 2. Thread: start = 9, pos = 1
            //
            // pos wird immer um zwei inkrementiert, so dass Thread 1 die
            // geraden und Thread 2 die ungeraden Stellen berechnet.
            int start = i * 9, step = this.threads.Length * 9;
            int pos = i;

            // Den Timer brauchen wir nur im ersten Thread
            if (i == 0)
                timer = new Timer();
            for (int j = start; j < end; j += step)
            {
                // Nur für den ersten Thread
                if (i == 0)
                {
                    // Progressbar aktualisieren
                    this.parent.SetProgressBarValue(end, j);
                    // Für die Berechnung die Zeit stoppen
                    timer.Start();
                    pi.CalcDigit(j + 1);
                    timer.Stop();
                    // Zeit inkrementieren
                    this.time += timer.Duration;
                    // Aktuelle Zeit ausgeben
                    this.parent.SetElapsedLabelText(string.Format("{0:#0.0000}", time));
                }

                // Für die restlichen Thread
                if (i != 0) pi.CalcDigit(j + 1);
                // Ergebnis speichern
                this.result[pos] = pi.Result;
                // Position inkrementieren
                pos += this.threads.Length;
            }
        }
    }
}