using System;
using System.Drawing;
using System.Windows.Forms;

namespace PI
{
    /// <summary>
    /// GUI
    /// </summary>
    public partial class MyForm : Form
    {
        delegate void SetTextCallback(string text);
        delegate void SetButtonCallback();
        delegate void SetProgressCallback(long l, long i);

        private Controller controller;
        
        /// <summary>
        /// Konstruktor
        /// </summary>
        public MyForm()
        {
            InitializeComponent();
        }

        private void calcButton_Click(object sender, EventArgs e)
        {
            this.ToggleButtons();
            this.outBox.Text = "3." + Environment.NewLine;
            // Sollte ein Vielfach von 9 sein.
            this.digitsBox.Value -= (this.digitsBox.Value % 9);

            this.controller = new Controller(this, (int)this.digitsBox.Value);
            this.controller.StartCalculation(this.multiCheck.Checked);
        }

        /// <summary>
        /// Die Button (de)aktivieren
        /// </summary>
        public void ToggleButtons()
        {
            if (this.cancelButton.InvokeRequired)
            {
                SetButtonCallback t = new SetButtonCallback(this.ToggleButtons);
                this.Invoke(t);
                return;
            }

            this.cancelButton.Enabled = !this.cancelButton.Enabled;
            this.calcButton.Enabled = !this.calcButton.Enabled;
            this.digitsBox.Enabled = !this.digitsBox.Enabled;
            this.multiCheck.Enabled = !this.multiCheck.Enabled;
        }

        /// <summary>
        /// Wert fr die Progressbar setzen
        /// </summary>
        /// <param name="max">Maximalwert</param>
        /// <param name="i">Aktueller Wert</param>
        public void SetProgressBarValue(long max, long i)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new SetProgressCallback(this.SetProgressBarValue),
                    new object[] { max, i });
                return;
            }
            double v = ((double)i / (double)max) * 100;
            this.progressBar.Value = (int)v;

        }

        /// <summary>
        /// Text fr das "Vergangene Zeit-Label" setzen
        /// </summary>
        /// <param name="d">Wert</param>
        public void SetElapsedLabelText(string d)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new SetTextCallback(this.SetElapsedLabelText),
                    new object[] { d });
                return;
            }
            
            this.elapsedLabel.Text = d.ToString();
        }

        /// <summary>
        /// Text fr die Ausgabe-Textbox setzen
        /// </summary>
        /// <param name="s">Wert</param>
        public void AppendOutBoxText(string s)
        {
            if (this.outBox.InvokeRequired)
            {
                SetTextCallback t = new SetTextCallback(this.AppendOutBoxText);
                this.Invoke(t, new object[] { s });
                return;
            }
            
            this.outBox.AppendText(s.ToString() + "\t");
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.controller.Abort();
            this.ToggleButtons();
        }
    }
}