using System;

namespace PI
{
    /// <summary>
    /// http://fabrice.bellard.free.fr/pi
    /// </summary>
    public class PI
    {
        private string result;
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse PI
        /// Mit der Methode CalcDigit wird der Vorgang gestartet.
        /// </summary>
        public PI()
        {
            this.result = "";
        }
        /// <summary>
        /// (a*b) mod m
        /// </summary>
        /// <param name="a">Erste Zahl</param>
        /// <param name="b">Zweite Zahl</param>
        /// <param name="m">Mod</param>
        /// <returns>Das Ergebnis aus (a*b) % m</returns>
        private int MulMod(int a, int b, int m)
        {
            return (int)(((int)a * (int)b) % m);
        }
        /// <summary>
        /// Invertiert x mod y
        /// </summary>
        /// <param name="x">Erste Zahl</param>
        /// <param name="y">Zweite Zahl</param>
        /// <returns>Invertierstes Ergebnis aus x % y</returns>
        private int InvMod(int x, int y)
        {
            int q, u, v, a, c, t;

            u = x;
            v = y;
            c = 1;
            a = 0;

            do
            {
                q = v / u;

                t = c;
                c = a - q * c;
                a = t;

                t = u;
                u = v - q * u;
                v = t;
            } while (u != 0);

            a = a % y;

            if (a < 0) a = y + a;

            return a;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="u"></param>
        /// <param name="v"></param>
        /// <returns></returns>
        private int InvMod2(int u, int v)
        {
            bool flag = false;
            int u1, u3, v1, v3, t1, t3;

            u1 = 1;
            u3 = u;

            v1 = v;
            v3 = v;

            if ((u & 1) != 0)
            {
                t1 = 0;
                t3 = -v;
                flag = true;
            }
            else
            {
                t1 = 1;
                t3 = u;
            }

            do
            {
                do
                {
                    if (!flag)
                    {
                        if ((t1&1)==0)
                        {
                            t1 = t1 >> 1;
                            t3 = t3 >> 1;
                        }
                        else
                        {
                            t1 = (t1 + v) >> 1;
                            t3 = t3 >> 1;
                        }
                    }
                    flag = false;
                } while ((t3 & 1) == 0);

                if (t3 >= 0)
                {
                    u1 = t1;
                    u3 = t3;
                }
                else
                {
                    v1 = v - t1;
                    v3 =- t3;
                }

                t1 = u1 - v1;
                t3 = u3 - v3;
                if (t1 < 0) t1 = t1 + v;
            } while (t3 != 0);
            
            return u1;
        }
        /// <summary>
        /// (a^b) mod m
        /// </summary>
        /// <param name="a">Erste Zahl</param>
        /// <param name="b">Zweite Zahl</param>
        /// <param name="m">Mod</param>
        /// <returns>Ergebnis aus (a^b) % m</returns>
        private int PowMod(int a, int b, int m)
        {
            int r, aa;
            r = 1;
            aa = a;

            while (true)
            {
                if ((b & 1) != 0) r = MulMod(r, aa, m);
                b = b >> 1;
                if (b == 0) break;
                aa = MulMod(aa, aa, m);
            }

            return r;
        }
        /// <summary>
        /// berprft, ob es sich bei der bergebenen Zahl
        /// um eine Primzahl handelt
        /// </summary>
        /// <param name="n">Zahl</param>
        /// <returns>true, wenn Primzahl, sonst false</returns>
        private bool IsPrime(int n)
        {
            if ((n % 2) == 0) return false;

            int r = (int)Math.Sqrt(n);

            for (int i = 3; i <= r; i += 2)
                if ((n % i) == 0)
                    return false;

            return true;
        }

        /// <summary>
        /// Gibt die nchste Primzahl nach der bergebenen Zahl wieder
        /// </summary>
        /// <param name="n">Zahl</param>
        /// <returns>Nchste Primzahl</returns>
        private int NextPrime(int n)
        {
            do
            {
                n++;
            } while (!IsPrime(n));

            return n;
        }
        private void DIVN(ref int t, ref int a, ref int v, int vinc, ref int kq, int kqinc)
        {
            kq += kqinc;
            if (kq >= a)
            {
                do { kq -= a; } while (kq >= a);
                if (kq == 0)
                {
                    do
                    {
                        t = t / a;
                        v += vinc;
                    } while ((t % a) == 0);
                }
            }
        }
        /// <summary>
        /// Gibt das Teilergebnis wieder
        /// </summary>
        public string Result
        {
            get { return this.result; }
        }
        /// <summary>
        /// Berechnet die n-te Stelle von PI
        /// </summary>
        /// <param name="n">die n-te Stelle</param>
        public void CalcDigit(int n)
        {
            int av, a, vmax, num, den, k, kq1, kq2, kq3, kq4, t, v, s, i, t1;
            int numBits = (int)((n + 20) * Math.Log(10) / Math.Log(13.5));
            double sum = 0;

            for (a = 2; a <= (3 * numBits); a = this.NextPrime(a))
            {
                vmax = (int)(Math.Log(3 * numBits) / Math.Log(a));

                if (a == 2)
                {
                    vmax = vmax + (numBits - n);
                    if (vmax <= 0) continue;
                }
                
                av = 1;

                for (i = 0; i < vmax; i++) av = av * a;

                s = 0;
                den = 1;
                kq1 = 0;
                kq2 = -1;
                kq3 = -3;
                kq4 = -2;

                if (a == 2)
                {
                    num = 1;
                    v = -n;
                }
                else
                {
                    num = this.PowMod(2, n, av);
                    v = 0;
                }

                for (k = 1; k <= numBits; k++)
                {
                    t = 2 * k;
                    this.DIVN(ref t, ref a, ref v, -1, ref kq1, 2);
                    num = this.MulMod(num, t, av);

                    t = 2 * k - 1;
                    this.DIVN(ref t, ref a, ref v, -1, ref kq2, 2);
                    num = this.MulMod(num, t, av);

                    t = 3 * (3 * k - 1);
                    this.DIVN(ref t, ref a, ref v, 1, ref kq3, 9);
                    den = this.MulMod(den, t, av);

                    t = 3 * k - 2;
                    this.DIVN(ref t, ref a, ref v, 1, ref kq4, 3);

                    if (a != 2) t = t * 2;
                    else v++;

                    den = this.MulMod(den, t, av);

                    if (v > 0)
                    {
                        if (a != 2) t = this.InvMod2(den, av);
                        else t = this.InvMod(den, av);

                        t = this.MulMod(t, num, av);
                        for (i = v; i < vmax; i++) t = this.MulMod(t, a, av);
                        t1 = 25 * k - 3;
                        t = this.MulMod(t, t1, av);
                        s += t;
                        if (s >= av) s -= av;
                    }
                }

                t = this.PowMod(5, n - 1, av);
                s = this.MulMod(s, t, av);
                sum = (sum + (double)s / (double)av) % 1.0;
            }

            int res = (int)(sum * 1e9);
            this.result = string.Format("{0:D9}", res);
        }
    }
}
